@@ -3,6 +3,7 @@ from dataclasses import dataclass, field
 from datetime import datetime
 from pathlib import Path
 from typing import (
+    TYPE_CHECKING,
     Any,
     Collection,
     Dict,
@@ -47,7 +48,7 @@ from ldb.index.utils import (
     separate_storage_and_non_storage_files,
     validate_locations_in_storage,
 )
-from ldb.params import ParamConfig
+from ldb.params import ParamConfig, ParamFunc
 from ldb.path import InstanceDir
 from ldb.storage import StorageLocation
 from ldb.typing import JSONDecoded, JSONObject
@@ -61,6 +62,9 @@ from ldb.utils import (
     write_data_file,
 )
 
+if TYPE_CHECKING:
+    from ldb.add import AddResult
+
 ENDING_DOUBLE_STAR_RE = r"(?:/+\*\*)+/*$"
 
 
@@ -71,6 +75,8 @@ class IndexedObjectResult(NamedTuple):
     new_annotation: bool
     new_data_object_path: bool
     data_object_hash: str
+    annotation_hash: str
+    transform_hashes: Optional[Sequence[str]]
 
 
 @dataclass
@@ -80,7 +86,11 @@ class IndexingResult:
     num_new_data_objects: int = 0
     num_new_annotations: int = 0
     num_new_data_object_paths: int = 0
-    data_object_hashes: List[str] = field(default_factory=list)
+    collection: Dict[str, str] = field(
+        default_factory=dict,
+    )
+    transforms: Dict[str, Sequence[str]] = field(default_factory=dict)
+    add_result: Optional["AddResult"] = None
 
     def append(self, item: IndexedObjectResult) -> None:
         self.num_found_data_objects += item.found_data_object
@@ -88,20 +98,29 @@ class IndexingResult:
         self.num_new_data_objects += item.new_data_object
         self.num_new_annotations += item.new_annotation
         self.num_new_data_object_paths += item.new_data_object_path
-        self.data_object_hashes.append(item.data_object_hash)
+        self.collection[item.data_object_hash] = item.annotation_hash
+        if item.transform_hashes is not None:
+            self.transforms[item.data_object_hash] = item.transform_hashes
 
     def summary(self, finished: bool = True) -> str:
         if finished:
             heading = "Finished indexing"
         else:
             heading = "Unable to finish indexing. Partial indexing results"
+
+        if self.add_result is None:
+            ending = ""
+        else:
+            ending = f"\n\n{self.add_result.summary()}"
         return (
             f"\n{heading}:\n"
             f"  Found data objects:    {self.num_found_data_objects:9d}\n"
             f"  Found annotations:     {self.num_found_annotations:9d}\n"
             f"  New data objects:      {self.num_new_data_objects:9d}\n"
             f"  New annotations:       {self.num_new_annotations:9d}\n"
-            f"  New data object paths: {self.num_new_data_object_paths:9d}"
+            f"  New data object paths: {self.num_new_data_object_paths:9d}\n"
+            f"  Transform sets:        {len(self.transforms):9d}"
+            f"{ending}"
         )
 
 
@@ -112,7 +131,9 @@ class Preprocessor(ParamConfig):
         storage_locations: Sequence[StorageLocation],
         params: Mapping[str, str],
         fmt: str,
+        param_processors: Optional[Mapping[str, Optional[ParamFunc]]] = None,
     ) -> None:
+        super().__init__(param_processors=param_processors)
         self.paths: List[str] = list(paths)
         self.storage_locations = storage_locations
         self.params = self.process_params(params, fmt)
@@ -151,11 +172,13 @@ class Indexer(ABC):
         preprocessor: Preprocessor,
         tags: Collection[str] = (),
         annot_merge_strategy: AnnotMergeStrategy = AnnotMergeStrategy.REPLACE,
+        ephemeral_remote: bool = False,
     ) -> None:
         self.ldb_dir = ldb_dir
         self.preprocessor = preprocessor
         self.tags = tags
         self.annot_merge_strategy = annot_merge_strategy
+        self.ephemeral_remote = ephemeral_remote
         self.result = IndexingResult()
         self.hashes: Dict[AbstractFileSystem, Dict[str, str]] = {}
 
@@ -182,8 +205,15 @@ class PairIndexer(Indexer):
         strict_format: bool,
         tags: Collection[str] = (),
         annot_merge_strategy: AnnotMergeStrategy = AnnotMergeStrategy.REPLACE,
+        ephemeral_remote: bool = False,
     ) -> None:
-        super().__init__(ldb_dir, preprocessor, tags, annot_merge_strategy)
+        super().__init__(
+            ldb_dir,
+            preprocessor,
+            tags,
+            annot_merge_strategy,
+            ephemeral_remote,
+        )
         self.read_any_cloud_location = read_any_cloud_location
         self.strict_format = strict_format
         self.old_to_new_files: Dict[
@@ -206,16 +236,26 @@ class PairIndexer(Indexer):
         self,
     ) -> Tuple[IndexingJobMapping, FSPathsMapping]:
         storage_locations = self.preprocessor.storage_locations
-        local_files, cloud_files = separate_local_and_cloud_files(
-            self.preprocessor.data_object_paths,
-        )
+        if self.ephemeral_remote:
+            possibly_ephemeral_files = self.preprocessor.data_object_paths
+            storage_only_files: FSPathsMapping = {}
+        else:
+            (
+                possibly_ephemeral_files,
+                storage_only_files,
+            ) = separate_local_and_cloud_files(
+                self.preprocessor.data_object_paths,
+            )
         if not self.read_any_cloud_location:
-            validate_locations_in_storage(cloud_files, storage_locations)
+            validate_locations_in_storage(
+                storage_only_files,
+                storage_locations,
+            )
         (
             local_storage_files,
             ephemeral_files,
         ) = separate_storage_and_non_storage_files(
-            local_files,
+            possibly_ephemeral_files,
             storage_locations,
         )
 
@@ -236,8 +276,9 @@ class PairIndexer(Indexer):
             ephemeral_files,
         )
         files = {
-            fs: cloud_files.get(fs, []) + local_storage_files.get(fs, [])
-            for fs in cloud_files.keys() | local_storage_files.keys()
+            fs: storage_only_files.get(fs, [])
+            + local_storage_files.get(fs, [])
+            for fs in storage_only_files.keys() | local_storage_files.keys()
         }
         annotation_paths = {
             fs: paths.copy()
@@ -423,7 +464,7 @@ class IndexingItem(ABC):
 
     @cached_property
     def raw_annotation_content(self) -> JSONDecoded:
-        raise NotImplementedError
+        return None
 
     @cached_property
     def annotation_content(self) -> JSONDecoded:
@@ -489,6 +530,8 @@ class IndexingItem(ABC):
             new_annotation=new_annotation,
             new_data_object_path=self.found_new_data_object_path,
             data_object_hash=self.data_object_hash,
+            annotation_hash=self.annotation_hash,
+            transform_hashes=None,
         )
 
     def data_object_to_write(self) -> List[DataToWrite]:
@@ -563,7 +606,7 @@ class AnnotationFileIndexingItem(IndexingItem):
     @cached_property
     def raw_annotation_content(self) -> JSONDecoded:
         if self.annotation_fsp is None:
-            raise IndexingException("Missing annotation_fsp")
+            return None
         return get_annotation_content(
             *self.annotation_fsp,
         )
