@@ -29,6 +29,7 @@ from ldb.data_formats import INSTANTIATE_FORMATS, Format
 from ldb.dataset import OpDef, get_annotation
 from ldb.exceptions import LDBException
 from ldb.fs.utils import FSProtocol, first_protocol, unstrip_protocol
+from ldb.index.annotation_only import AnnotOnlyParamConfig
 from ldb.index.inferred import InferredParamConfig
 from ldb.params import ParamConfig
 from ldb.path import InstanceDir, WorkspacePath
@@ -93,6 +94,8 @@ def instantiate(
 
     if fmt == Format.INFER:
         processed_params = InferredParamConfig().process_params(params, fmt)
+    elif fmt == Format.ANNOT:
+        processed_params = AnnotOnlyParamConfig().process_params(params, fmt)
     else:
         processed_params = ParamConfig().process_params(params, fmt)
 
@@ -102,7 +105,8 @@ def instantiate(
         paths,
         query_args,
         warn=warn,
-        include_transforms=fmt in (Format.STRICT, Format.BARE),
+        include_transforms=fmt
+        in (Format.STRICT, Format.BARE, Format.ANNOT, Format.INFER),
     )
     return instantiate_collection(
         ldb_dir,
@@ -162,7 +166,7 @@ def instantiate_collection(
                     fmt,
                 )
             if apply:
-                paths = [str(ldb_dir / InstanceDir.USER_FILTERS)]
+                paths = [str(ldb_dir / InstanceDir.USER_TRANSFORMS)]
                 apply_transform(
                     apply,
                     final_tmp_dir,
@@ -200,6 +204,12 @@ def instantiate_collection_directly(
             fmt == Format.STRICT,
         )
     if fmt == Format.ANNOT:
+        single_file = config.params.get("single-file", False)
+        if single_file:
+            return copy_single_annot(
+                config,
+                collection,
+            )
         return copy_annot(
             config,
             collection,
@@ -376,6 +386,8 @@ class PairInstItem(RawPairInstItem):
         return self.config.intermediate_dir
 
     def instantiate(self) -> ItemCopyResult:
+        # TODO: include in copy_result transform output files, not just
+        # raw instantiated files
         copy_result = self.copy_files()
         data = {
             "output_dir": self.config.dest_dir,
@@ -420,7 +432,14 @@ class PairInstItem(RawPairInstItem):
                 data_str = json.dumps(
                     {"transform_name": info.name, **data},
                 )
-                pipe_to_proc(data_str, info.transform.value)
+                pipe_to_proc(
+                    data_str,
+                    info.transform.value,
+                    paths=[
+                        str(self.config.ldb_dir / InstanceDir.USER_TRANSFORMS),
+                    ],
+                    set_cwd=True,
+                )
             else:
                 raise ValueError(
                     f"Invalid transform type: {info.transform.transform_type}",
@@ -430,6 +449,8 @@ class PairInstItem(RawPairInstItem):
 
 @dataclass
 class AnnotationOnlyInstItem(RawPairInstItem):
+    transform_infos: Optional[Collection[TransformInfo]] = None
+
     @cached_property
     def annotation_content(self) -> JSONDecoded:
         annotation: JSONObject = get_annotation(  # type: ignore[assignment]
@@ -448,12 +469,25 @@ class AnnotationOnlyInstItem(RawPairInstItem):
             },
             "annotation": annotation,
         }
+        if self.transform_infos:
+            annotation["ldb-meta"] = {
+                "transforms": [t.to_dict() for t in self.transform_infos],
+            }
         return annotation
 
     def copy_files(self) -> ItemCopyResult:
         return ItemCopyResult(annotation=self.copy_annotation())
 
 
+@dataclass
+class SingleAnnotationInstItem(AnnotationOnlyInstItem):
+    annotation_list: List[JSONDecoded] = field(default_factory=list)
+
+    def copy_annotation(self) -> str:
+        self.annotation_list.append(self.annotation_content)
+        return ""
+
+
 @dataclass
 class InferInstItem(RawPairInstItem):
     annotation_hash: str
@@ -462,25 +496,11 @@ class InferInstItem(RawPairInstItem):
 
     @cached_property
     def base_dest(self) -> str:
-        parts: List[str] = []
-        key: str
-        label: Union[JSONObject, str]
-        try:
-            label = self.annotation_content  # type: ignore[assignment]
-            for key in self.label_key:
-                label = label[key]  # type: ignore[index]
-        except Exception as exc:
-            # TODO: convert self.label_key to jmespath expression for message
-            raise LDBException(
-                "Annotations for tensorflow-inferred format should contain "
-                f"key: {self.label_key}",
-            ) from exc
-
-        if label != self.base_label:
-            while isinstance(label, dict):
-                key, label = next(iter(label.items()))  # type: ignore[assignment] # noqa: E501
-                parts.append(key)
-            parts.append(label)
+        parts = infer_dir(
+            self.annotation_content,
+            self.label_key,
+            self.base_label,
+        )
         return os.path.join(
             self.dest_dir,
             *parts,
@@ -600,6 +620,7 @@ def copy_annot(
                     config,
                     data_object_hash,
                     annotation_hash,
+                    config.transform_infos.get(data_object_hash),
                 ),
             )
     _, annot_paths = instantiate_items(items)
@@ -611,6 +632,36 @@ def copy_annot(
     )
 
 
+def copy_single_annot(
+    config: InstConfig,
+    collection: Mapping[str, Optional[str]],
+) -> InstantiateResult:
+    items = []
+    annotation_list: List[JSONDecoded] = []
+    for data_object_hash, annotation_hash in collection.items():
+        if annotation_hash:
+            items.append(
+                SingleAnnotationInstItem(
+                    config,
+                    data_object_hash,
+                    annotation_hash,
+                    config.transform_infos.get(data_object_hash),
+                    annotation_list=annotation_list,
+                ),
+            )
+    instantiate_items(items)
+    annotation = serialize_annotation(annotation_list)
+    annotation_bytes = annotation.encode()
+    dest = Path(os.path.join(config.dest_dir, "dataset.json"))
+    write_data_file(dest, annotation_bytes)
+    return InstantiateResult(
+        [],
+        [],
+        0,
+        len(annotation_list),
+    )
+
+
 def copy_infer(
     config: InstConfig,
     collection: Mapping[str, Optional[str]],
@@ -625,11 +676,30 @@ def copy_infer(
                 "Missing annotation for data object: "
                 f"{DATA_OBJ_ID_PREFIX}{data_object_hash}",
             )
-    items = []
-    for data_object_hash, annotation_hash in cast(
-        Mapping[str, str],
+    collection = cast(Mapping[str, str], collection)
+    if config.transform_infos:
+        return _copy_infer_with_transforms(
+            config,
+            collection,
+            label_key,
+            base_label,
+        )
+    return _copy_infer_without_transforms(
+        config,
         collection,
-    ).items():
+        label_key,
+        base_label,
+    )
+
+
+def _copy_infer_without_transforms(
+    config: InstConfig,
+    collection: Mapping[str, str],
+    label_key: Sequence[str],
+    base_label: str,
+) -> InstantiateResult:
+    items = []
+    for data_object_hash, annotation_hash in collection.items():
         items.append(
             InferInstItem(
                 config,
@@ -648,6 +718,91 @@ def copy_infer(
     )
 
 
+def _copy_infer_with_transforms(
+    config: InstConfig,
+    collection: Mapping[str, str],
+    label_key: Sequence[str],
+    base_label: str,
+) -> InstantiateResult:
+    result = copy_pairs(
+        config,
+        collection,
+        strict=False,
+    )
+
+    collection = {}
+    annotation_paths = set()
+    data_object_paths = set()
+    for path in os.listdir(config.dest_dir):
+        if path.endswith(".json"):
+            annotation_paths.add(os.path.join(config.dest_dir, path))
+        else:
+            data_object_paths.add(os.path.join(config.dest_dir, path))
+
+    for path in data_object_paths:
+        base, _ = os.path.splitext(path)
+        annot_path = f"{base}.json"
+        if annot_path in annotation_paths:
+            collection[path] = annot_path
+
+    pairs_to_inferred_dirs(
+        str(config.dest_dir),
+        collection.items(),
+        label_key,
+        base_label,
+    )
+    for path in annotation_paths | (data_object_paths - collection.keys()):
+        os.unlink(path)
+    return result
+
+
+def pairs_to_inferred_dirs(
+    dir_path: str,
+    path_mapping: Iterable[Tuple[str, str]],
+    label_key: Sequence[str],
+    base_label: str = "",
+) -> None:
+    for data_object_path, annotation_path in path_mapping:
+        with open(annotation_path, encoding="utf-8") as f:
+            raw_content = f.read()
+        annot = json.loads(raw_content)
+        path_parts = infer_dir(annot, label_key, base_label)
+        dest = os.path.join(
+            dir_path,
+            *path_parts,
+            os.path.basename(data_object_path),
+        )
+        os.makedirs(os.path.dirname(dest), exist_ok=True)
+        shutil.move(data_object_path, dest)
+
+
+def infer_dir(
+    annotation: JSONDecoded,
+    label_key: Sequence[str],
+    base_label: str = "",
+) -> List[str]:
+    parts: List[str] = []
+    key: str
+    label: Union[JSONObject, str]
+    try:
+        label = annotation  # type: ignore[assignment]
+        for key in label_key:
+            label = label[key]  # type: ignore[index]
+    except Exception as exc:
+        # TODO: convert self.label_key to jmespath expression for message
+        raise LDBException(
+            "Annotations for tensorflow-inferred format should contain "
+            f"key: {label_key}",
+        ) from exc
+
+    if label != base_label:
+        while isinstance(label, dict):
+            key, label = next(iter(label.items()))  # type: ignore[assignment] # noqa: E501
+            parts.append(key)
+        parts.append(label)
+    return parts
+
+
 def copy_label_studio(
     config: InstConfig,
     collection: Mapping[str, Optional[str]],
